@@ -580,30 +580,126 @@ If no such buffer is found, report a user-error."
     (unless has-test-buffer
       (user-error "No test file found in current windows. Please open a test file first"))))
 
+(defconst ai-code--tdd-test-pattern-instruction
+  "\nFollow the test-code pattern in the current project. Write the test-code in the test-file. If the test-file does not exist, create it using the same test-filename pattern used in this repository."
+  "Instruction appended to TDD prompts to enforce the project's test pattern.")
+
+(defconst ai-code--tdd-run-test-after-this-stage-instruction
+  " Run test after this stage and output the summary of test result. List the public API / log key / config key change if there is."
+  "Instruction appended to single-stage TDD prompts.")
+
+(defconst ai-code--tdd-run-test-after-each-stage-instruction
+  " Run test after each stage and output the summary of test result. List the public API / log key / config key change if there is."
+  "Instruction appended to multi-stage TDD prompts.")
+
+(defconst ai-code--tdd-red-green-base-instruction
+  " Follow TDD principles - write the failing test first, then implement the minimal code to make it pass"
+  "Base instruction shared by Red+Green style TDD prompts.")
+
+(defconst ai-code--tdd-red-green-tail-instruction
+  " Only update test and source code. Run the tests and follow up with the test result (fix code if there is error)."
+  "Trailing instruction shared by Red+Green style TDD prompts.")
+
+(defconst ai-code--tdd-with-refactoring-extension-instruction
+  ". After that, refactor only the code you just changed. In refactor staging, first review the code diff (including tests) and identify the highest-impact cleanup. Then apply focused refactoring that preserves behavior while improving readability, keeping classes/functions small and cohesive / easy to test, reducing duplication, and simplifying naming and control flow."
+  "Refactoring extension shared by Red+Green+Blue style TDD prompts.")
+
 (defun ai-code--tdd-red-stage (function-name)
   "Handle the Red stage of TDD for FUNCTION-NAME: Write a failing test."
-  (if (and (region-active-p) (not (derived-mode-p 'prog-mode)))
-      ;; If there is a selected region, and it is not a prog-mode derived buffer,
-      ;; assume this is the information / exception of failed test.
-      ;; Use it as initial prompt to fix the source code.
-      (let* ((failure-info (buffer-substring-no-properties (region-beginning) (region-end)))
-             (prompt (ai-code-read-string "Confirm test failure to fix: " failure-info))
+  (let ((test-pattern-instruction ai-code--tdd-test-pattern-instruction))
+    (if (and (region-active-p) (not (derived-mode-p 'prog-mode)))
+        ;; If there is a selected region, and it is not a prog-mode derived buffer,
+        ;; assume this is the information / exception of failed test.
+        ;; Use it as initial prompt to fix the source code.
+        (let* ((failure-info (buffer-substring-no-properties (region-beginning) (region-end)))
+               (prompt (ai-code-read-string "Confirm test failure to fix: " failure-info))
+               (file-info (ai-code--get-context-files-string))
+               (tdd-instructions (format "Fix the code to resolve the following error:\n%s%s%s"
+                                         prompt
+                                         file-info
+                                         test-pattern-instruction)))
+          (ai-code--insert-prompt tdd-instructions))
+      ;; Original path: write a failing test
+      (ai-code--ensure-test-buffer-visible)
+      (let* ((feature-desc (ai-code-read-string
+                            (if function-name
+                                (format "Describe the feature to test for '%s': " function-name)
+                              "Describe the feature to test: ") "Implement test functions using test cases described in the comments."))
              (file-info (ai-code--get-context-files-string))
-             (tdd-instructions (format "Fix the code to resolve the following error:\n%s%s" prompt file-info)))
-        (ai-code--insert-prompt tdd-instructions))
-    ;; Original path: write a failing test
-    ;; DONE: at least one buffer in current window should be a test
-    ;; file (contains 'test' in name, case insensitive), otherwise
-    ;; report error and exit this function
-    (ai-code--ensure-test-buffer-visible)
-    (let* ((feature-desc (ai-code-read-string
-                          (if function-name
-                              (format "Describe the feature to test for '%s': " function-name)
-                            "Describe the feature to test: ") "Implement test functions using test cases described in the comments."))
-           (file-info (ai-code--get-context-files-string))
-           (tdd-instructions
-            (format "%s%s\nFollow TDD principles - write only the test now, not the implementation. The test should fail when run because the functionality doesn't exist yet. Only update test file code." feature-desc file-info)))
-      (ai-code--insert-prompt tdd-instructions))))
+             (tdd-instructions
+              (format "%s%s\nFollow TDD principles - write only the test now, not the implementation. The test should fail when run because the functionality doesn't exist yet. Only update test file code.%s"
+                      feature-desc
+                      file-info
+                      (concat ai-code--tdd-run-test-after-this-stage-instruction
+                              test-pattern-instruction))))
+        (ai-code--insert-prompt tdd-instructions)))))
+
+(defun ai-code--tdd-source-function-context-p (function-name)
+  "Return non-nil when FUNCTION-NAME is in a non-test source buffer."
+  (and function-name
+       buffer-file-name
+       (derived-mode-p 'prog-mode)
+       (let ((case-fold-search t))
+         (not (string-match-p "test" (file-name-nondirectory buffer-file-name))))))
+
+(defun ai-code--write-test (function-name)
+  "Write a test for FUNCTION-NAME in the corresponding test file."
+  (let* ((source-file (and buffer-file-name (file-name-nondirectory buffer-file-name)))
+         (test-file-hint (if source-file
+                             (format "the corresponding test file for %s, following test pattern of this repo"
+                                     source-file)
+                           "the corresponding test file"))
+         (write-test-desc (ai-code-read-string
+                           "Write test instruction: "
+                           (format "Write test for '%s' in %s."
+                                   function-name
+                                   test-file-hint)))
+         (file-info (ai-code--get-context-files-string))
+         (tdd-instructions
+          (format "%s%s\nFollow TDD principles - write only the test now, not the implementation. Only update test file code.%s"
+                  write-test-desc
+                  file-info
+                  ai-code--tdd-test-pattern-instruction)))
+    (ai-code--insert-prompt tdd-instructions)))
+
+(defun ai-code--tdd-red-green-stage (function-name)
+  "Handle the Red + Green stage for FUNCTION-NAME in one prompt."
+  ;; (ai-code--ensure-test-buffer-visible)
+  (let* ((feature-desc (ai-code-read-string
+                        (if function-name
+                            (format "Describe the feature to test for '%s': " function-name)
+                          "Describe the feature to test: ")
+                        "Implement test functions using test cases described in the comments."))
+         (file-info (ai-code--get-context-files-string))
+         (tdd-instructions
+          (format "%s%s\n%s%s%s%s"
+                  feature-desc
+                  file-info
+                  ai-code--tdd-red-green-base-instruction
+                  ai-code--tdd-red-green-tail-instruction
+                  ai-code--tdd-run-test-after-each-stage-instruction
+                  ai-code--tdd-test-pattern-instruction)))
+    (ai-code--insert-prompt tdd-instructions)))
+
+(defun ai-code--tdd-red-green-blue-stage (function-name)
+  "Handle the Red + Green + Blue stage for FUNCTION-NAME in one prompt."
+  ;; (ai-code--ensure-test-buffer-visible)
+  (let* ((feature-desc (ai-code-read-string
+                        (if function-name
+                            (format "Describe the feature to test for '%s': " function-name)
+                          "Describe the feature to test: ")
+                        "Implement test functions using test cases described in the comments."))
+         (file-info (ai-code--get-context-files-string))
+         (tdd-instructions
+          (format "%s%s\n%s%s%s%s%s"
+                  feature-desc
+                  file-info
+                  ai-code--tdd-red-green-base-instruction
+                  ai-code--tdd-with-refactoring-extension-instruction
+                  ai-code--tdd-red-green-tail-instruction
+                  ai-code--tdd-run-test-after-each-stage-instruction
+                  ai-code--tdd-test-pattern-instruction)))
+    (ai-code--insert-prompt tdd-instructions)))
 
 (defun ai-code--tdd-green-stage (function-name)
   "Handle the Green stage of TDD for FUNCTION-NAME: Make the test pass.
@@ -622,19 +718,38 @@ to fix code."
          (implementation-desc (ai-code-read-string "Implementation instruction: " initial-input))
          (file-info (ai-code--get-context-files-string))
          (tdd-instructions
-          (format "%s%s\nFollow TDD principles - implement the code needed to make the test pass."
-                  implementation-desc file-info)))
+          (format "%s%s\nFollow TDD principles - implement the code needed to make the test pass.%s"
+                  implementation-desc
+                  file-info
+                  ai-code--tdd-run-test-after-this-stage-instruction)))
     (ai-code--insert-prompt tdd-instructions)))
 
 (defun ai-code--run-test-ai-assisted ()
   "Run tests by sending a prompt to AI with current context."
-  (let* ((function-name (which-function))
-         (file-info (ai-code--get-context-files-string))
-         (initial-input (format "Run the tests for the current %s using appropriate test_runner.%s"
-                         (if function-name
-                             (format "function '%s'" function-name)
-                           "file")
-                         file-info))
+  (let* ((is-dired (derived-mode-p 'dired-mode))
+         (function-name (unless is-dired (which-function)))
+         (file-info (unless is-dired (ai-code--get-context-files-string)))
+         (error-handling-instruction
+          (concat "\n\nIf any test fails:"
+                  "\n1. Analyze the test failure output carefully to identify which test(s) failed"
+                  "\n2. Investigate the root cause by examining the test code and related source files"
+                  "\n3. Provide a clear explanation of what went wrong"
+                  "\n4. Suggest specific code fixes for user approval before making any changes"))
+         (initial-input
+          (cond
+           (is-dired
+            (format "Run the tests for source code in directory '%s' using appropriate test runner.%s"
+                    (dired-current-directory)
+                    error-handling-instruction))
+           (function-name
+            (format "Run the tests for the current function '%s' using appropriate test runner.%s%s"
+                    function-name
+                    file-info
+                    error-handling-instruction))
+           (t
+            (format "Run the tests for the current file using appropriate test runner.%s%s"
+                    file-info
+                    error-handling-instruction))))
          (prompt (ai-code-read-string "Send to AI: " initial-input)))
     (ai-code--insert-prompt prompt)))
 
@@ -674,23 +789,36 @@ Helps users follow Kent Beck's TDD methodology with AI assistance.
 Works with both source code and test files that have been added to ai-code."
   (interactive)
   (let* ((function-name (which-function))
+         (use-write-test-stage (ai-code--tdd-source-function-context-p function-name))
+         (red-stage-label (if use-write-test-stage
+                              (format "1. Red (Write test for %s)" function-name)
+                            "1. Red (Write failing test)"))
          (cycle-stage (completing-read
                        "Select TDD stage: "
-                       '("0. Run unit-tests"
-                         "1. Red (Write failing test)"
-                         "2. Green (Make test pass)"
-                         "3. Refactor (Improve code quality)")
+                       (list "0. Run unit-tests"
+                             red-stage-label
+                             "2. Green (Make test pass)"
+                             "3. Blue (Refactor, improve code quality)"
+                             "4. Red + Green (One prompt)"
+                             "5. Red + Green + Blue (One prompt)")
                        nil t))
          (stage-num (string-to-number (substring cycle-stage 0 1))))
     (cond
      ;; Run tests
      ((= stage-num 0) (ai-code-run-test))
      ;; Red stage - write failing test
-     ((= stage-num 1) (ai-code--tdd-red-stage function-name))
+     ((= stage-num 1)
+      (if use-write-test-stage
+          (ai-code--write-test function-name)
+        (ai-code--tdd-red-stage function-name)))
      ;; Green stage - make test pass
      ((= stage-num 2) (ai-code--tdd-green-stage function-name))
      ;; Refactor stage - call the main refactoring function in TDD mode
-     ((= stage-num 3) (ai-code-refactor-book-method t)))))
+     ((= stage-num 3) (ai-code-refactor-book-method t))
+     ;; Red + Green combined in one prompt
+     ((= stage-num 4) (ai-code--tdd-red-green-stage function-name))
+     ;; Red + Green + Blue combined in one prompt
+     ((= stage-num 5) (ai-code--tdd-red-green-blue-stage function-name)))))
 
 (provide 'ai-code-agile)
 ;;; ai-code-agile.el ends here
