@@ -11,9 +11,9 @@
 
 (defvar yas-snippet-dirs)
 
-(declare-function magit-toplevel "magit" (&optional dir))
+(declare-function ai-code--git-root "ai-code-file" (&optional dir))
 
-(defvar ai-code-use-gptel-headline)
+(defvar ai-code-use-gptel-headline nil)
 (defvar ai-code-prompt-suffix)
 (defvar ai-code-auto-test-type)
 (defvar ai-code-auto-test-suffix)
@@ -29,6 +29,7 @@
 (declare-function ai-code--git-ignored-repo-file-p "ai-code-git" (file root))
 (declare-function ai-code--hash-completion-target-file "ai-code-input" (&optional end-pos))
 (declare-function ai-code--choose-symbol-from-file "ai-code-input" (file))
+(declare-function ai-code-read-string "ai-code-input" (prompt &optional initial-input candidate-list))
 (declare-function ai-code-current-backend-label "ai-code-backends" ())
 
 (defcustom ai-code-prompt-preprocess-filepaths t
@@ -150,6 +151,15 @@ Only works when gptel package is installed, otherwise shows error message."
   "Get the buffer for PROMPT-FILE, without selecting it."
   (find-file-noselect prompt-file))
 
+(defun ai-code--insert-backend-label-drawer ()
+  "Insert an Org drawer recording the current AI backend label."
+  (let ((label (condition-case nil
+                   (ai-code-current-backend-label)
+                 (error "unknown"))))
+    (insert ":PROPERTIES:\n")
+    (insert (format ":AGENT: %s\n" label))
+    (insert ":END:\n")))
+
 (defun ai-code--append-prompt-to-buffer (prompt-text)
   "Append formatted PROMPT-TEXT to the end of the current buffer.
 This includes generating a headline and formatting the prompt.
@@ -157,6 +167,7 @@ Returns the full prompt text with suffix for sending to AI."
   (goto-char (point-max))
   (insert "\n\n")
   (ai-code--generate-prompt-headline prompt-text)
+  (ai-code--insert-backend-label-drawer)
   (ai-code--format-and-insert-prompt prompt-text))
 
 (defun ai-code--send-prompt (full-prompt)
@@ -202,16 +213,15 @@ If WORD is a file path, it's converted to a relative path."
 
 (defun ai-code--preprocess-prompt-text (prompt-text)
   "Preprocess PROMPT-TEXT to replace file paths with relative paths.
-The function splits the prompt by whitespace, checks if each part is a file
-path within the current git repository, and if so, replaces it with a
-relative path prefixed with @.
+The function checks each non-whitespace token in the prompt; if a token is a
+file path within the current git repository it is replaced with a relative
+path prefixed with @.  Original whitespace is preserved.
 NOTE: This does not handle file paths containing spaces."
-  (if-let* ((git-root (magit-toplevel)))
-      (let ((git-root-truename (file-truename git-root)))
-        (mapconcat
-         (lambda (word) (ai-code--process-word-for-filepath word git-root-truename))
-         (split-string prompt-text "[ \t\n]+" t) ; split by whitespace and remove empty strings
-         " "))
+  (if-let* ((git-root-truename (ai-code--git-root)))
+      (replace-regexp-in-string
+       "[^ \t\n]+"
+       (lambda (word) (ai-code--process-word-for-filepath word git-root-truename))
+       prompt-text t t)
     ;; Not in a git repo, return original prompt
     prompt-text))
 
@@ -324,9 +334,8 @@ that root, otherwise return the absolute path."
 
 (defun ai-code--prompt-filepath-candidates ()
   "Return file path candidates for prompt completion."
-  (when-let ((git-root (magit-toplevel)))
-    (let* ((git-root-truename (file-truename git-root))
-           (current-file (buffer-file-name (current-buffer)))
+  (when-let ((git-root-truename (ai-code--git-root)))
+    (let* ((current-file (buffer-file-name (current-buffer)))
            (current-frame-dired-paths
             (ai-code--current-frame-dired-paths git-root-truename))
            (visible-files (ai-code--visible-window-files))
@@ -361,7 +370,7 @@ that root, otherwise return the absolute path."
 
 (defun ai-code--prompt-filepath-capf ()
   "Provide completion candidates for @file paths in prompt buffer."
-  (when (and (not (minibufferp)) (magit-toplevel))
+  (when (and (not (minibufferp)) (ai-code--git-root))
     (let ((end (point))
           (start (save-excursion
                    (skip-chars-backward "A-Za-z0-9_./-")
@@ -428,15 +437,46 @@ Special commands:
 The block is the text separated by blank lines.
 It trims leading/trailing whitespace."
   (interactive)
-  (let* ((block-text (thing-at-point 'paragraph))
+  ;; DONE: use save-excursion, mark-paragraph to get the block-text
+  (let* ((block-text (save-excursion
+                       (save-mark-and-excursion
+                         (ai-code--mark-prompt-block)
+                         (buffer-substring-no-properties (region-beginning)
+                                                         (region-end)))))
          (trimmed-text (when block-text (string-trim block-text))))
     (if (and trimmed-text (string-match-p "\\S-" trimmed-text))
-        (progn
-          (ai-code-cli-send-command trimmed-text)
-          (ai-code-cli-switch-to-buffer))
+        (if (and buffer-file-name
+                 (string= (file-name-nondirectory buffer-file-name)
+                          ai-code-prompt-file-name))
+            (ai-code--send-prompt trimmed-text)
+          (when-let ((edited-prompt
+                      (ai-code-read-string "Confirm and edit prompt before sending: "
+                                           trimmed-text)))
+            (ai-code--insert-prompt edited-prompt)))
       (message "No text in the current block to send."))))
 
-;; ai coding task feature
+(defun ai-code--mark-prompt-block ()
+  "Mark a code block. A code block is defined as multiple lines without empty lines inside,
+but with empty lines before and after the block."
+  (interactive)
+  (let ((start (point))
+        (end (point)))
+    (save-excursion
+      (while (and (not (bobp)) (not (looking-at-p "^$")))
+        (forward-line -1))
+      (unless (bobp)
+        (forward-line 1))
+      (setq start (point)))
+    (save-excursion
+      (while (and (not (eobp)) (not (looking-at-p "^$")))
+        (forward-line 1))
+      (setq end (point)))
+    (goto-char start)
+    (set-mark (point))
+    (goto-char end)
+    (message "Code block marked from line %d to line %d"
+             (line-number-at-pos start)
+             (line-number-at-pos end))))
 
 ;;;###autoload
 (defconst ai-code-files-dir-name ".ai.code.files"
@@ -454,7 +494,7 @@ based on the task name. Otherwise, use cleaned-up task name directly."
   "Get the task directory path.
 If in a git repository, return `.ai.code.files/` under git root.
 Otherwise, return the current `default-directory`."
-  (let ((git-root (magit-toplevel)))
+  (let ((git-root (ai-code--git-root)))
     (if git-root
         (expand-file-name ai-code-files-dir-name git-root)
       default-directory)))
@@ -502,13 +542,17 @@ Returns a filename with .org suffix."
   (insert (format "#+DATE: %s\n" (format-time-string "%Y-%m-%d")))
   (unless (string-empty-p task-url)
     (insert (format "#+URL: %s\n" task-url)))
+  (let ((branch (magit-get-current-branch)))
+    (when branch
+      (insert (format "#+BRANCH: %s\n" branch))))
   (let ((label (ai-code-current-backend-label)))
     (insert (format "#+AGENT: %s\n" label))
     (insert "#+SESSION_ID: <Usually you can get the session id with /status or /stat in AI coding window>\n"))
   (insert "\n* Task Description\n\n")
   (insert task-name)
   (insert "\n\n* Investigation\n\n")
-  (insert "# Enter your prompts here. After that,\n# Select them and use C-c a SPC (ai-code-send-command) to send to AI\n\n")
+  (insert "# Enter your prompts here. After that,\n# Select them and use C-c a SPC (ai-code-send-command) to send to AI\n")
+  (insert "#   Or You use C-c C-c (ai-code-prompt-send-block) to send the whole prompt block to AI\n\n")
   (insert "# Use C-c a n (ai-code-take-notes) to copy notes back from AI session\n\n")
   (insert "\n\n* Code Change\n\n"))
 
@@ -570,12 +614,7 @@ using GPTel, and creates the task file."
 
 ;;;###autoload
 (add-to-list 'auto-mode-alist
-             `(,(concat "^"
-                        (regexp-quote
-                         (file-name-as-directory
-                          (expand-file-name ai-code-files-dir-name)))
-                        ".*\\.org\\'")
-               . ai-code-prompt-mode))
+             '("/\\.ai\\.code\\.files/.*\\.org\\'" . ai-code-prompt-mode))
 
 (provide 'ai-code-prompt-mode)
 
