@@ -1,8 +1,8 @@
 ;;; ai-code.el --- Unified interface for AI coding backends such as Claude Code, Codex CLI, Gemini CLI, Copilot CLI, Opencode, Grok CLI, Aider CLI, etc -*- lexical-binding: t; -*-
 
 ;; Author: Kang Tu <tninja@gmail.com>
-;; Package-Version: 1.52
-;; Package-Revision: 74fd1c0b2db8
+;; Package-Version: 1.55
+;; Package-Revision: 19c761b9e18e
 ;; Package-Requires: ((emacs "28.1") (transient "0.8.0") (magit "2.1.0"))
 ;; URL: https://github.com/tninja/ai-code-interface.el
 
@@ -114,6 +114,7 @@
 (declare-function ai-code-open-backend-config "ai-code-backends")
 (declare-function ai-code-open-backend-agent-file "ai-code-backends")
 (declare-function ai-code-upgrade-backend "ai-code-backends")
+(declare-function ai-code-install-backend-skills "ai-code-backends")
 (declare-function ai-code-backends-infra--session-buffer-p "ai-code-backends-infra" (buffer))
 
 (declare-function ai-code--process-word-for-filepath "ai-code-prompt-mode" (word git-root-truename))
@@ -156,16 +157,24 @@ with a newline separator."
 
 (defun ai-code--test-after-code-change--resolve-tdd-suffix ()
   "Return the TDD-style suffix for test-after-code-change prompts."
-  (let ((pattern (when (and (boundp 'ai-code--tdd-test-pattern-instruction)
-                            (stringp ai-code--tdd-test-pattern-instruction))
-                   ai-code--tdd-test-pattern-instruction)))
-    (concat "Follow TDD principles - write the failing test first, then implement the minimal code to make it pass. Only update test and source code. Run the tests and follow up with the test result (fix code if there is error)."
-            (or pattern ""))))
+  (concat ai-code--tdd-red-green-base-instruction
+          ai-code--tdd-red-green-tail-instruction
+          ai-code--tdd-run-test-after-each-stage-instruction
+          ai-code--tdd-test-pattern-instruction))
+
+(defun ai-code--test-after-code-change--resolve-tdd-with-refactoring-suffix ()
+  "Return the TDD+refactoring suffix for test-after-code-change prompts."
+  (concat ai-code--tdd-red-green-base-instruction
+          ai-code--tdd-with-refactoring-extension-instruction
+          ai-code--tdd-red-green-tail-instruction
+          ai-code--tdd-run-test-after-each-stage-instruction
+          ai-code--tdd-test-pattern-instruction))
 
 (defconst ai-code--auto-test-type-ask-choices
   '(("Run tests after code change" . test-after-change)
     ("Test driven development: Write test first" . tdd)
-    ("Do not run tests" . nil))
+    ("Test driven development, follow up with refactoring" . tdd-with-refactoring)
+    ("Do not run tests" . no-test))
   "Choices for resolving the auto test suffix when `ai-code-auto-test-type` is `ask-me`.")
 
 (defun ai-code--read-auto-test-type-choice ()
@@ -195,7 +204,7 @@ Return one of: `code-change`, `non-code-change`, or `unknown`."
          (condition-case err
              (if (require 'gptel nil t)
                  (let* ((raw-answer (ai-code-call-gptel-sync
-                                     (concat "Classify whether this user prompt requests code changes in a repository.\n"
+                                     (concat "Classify whether this user prompt requests program code changes in a repository.\n"
                                              "Reply with exactly one token: CODE_CHANGE or NOT_CODE_CHANGE.\n"
                                              "Treat edit/refactor/implement/fix/add/remove/update/tests as CODE_CHANGE.\n"
                                              "Treat explain/summarize/discuss/review without editing as NOT_CODE_CHANGE.\n\n"
@@ -234,6 +243,12 @@ Return one of: `code-change`, `non-code-change`, or `unknown`."
                    'code-change)
            'tdd)
        'tdd))
+    ('tdd-with-refactoring
+     (if ai-code-use-gptel-classify-prompt
+         (when (eq (ai-code--gptel-classify-prompt-code-change prompt-text)
+                   'code-change)
+           'tdd-with-refactoring)
+       'tdd-with-refactoring))
     (_ nil)))
 
 (defun ai-code--auto-test-suffix-for-type (type)
@@ -241,6 +256,8 @@ Return one of: `code-change`, `non-code-change`, or `unknown`."
   (pcase type
     ('test-after-change ai-code-test-after-code-change-suffix)
     ('tdd (ai-code--test-after-code-change--resolve-tdd-suffix))
+    ('tdd-with-refactoring (ai-code--test-after-code-change--resolve-tdd-with-refactoring-suffix))
+    ('no-test "Do not run any test.")
     (_ nil)))
 
 (defun ai-code--resolve-auto-test-suffix-for-send (&optional prompt-text)
@@ -276,6 +293,7 @@ Return one of: `code-change`, `non-code-change`, or `unknown`."
   "Select how prompts request tests after code changes."
   :type '(choice (const :tag "Use test after code change prompt" test-after-change)
                  (const :tag "Use TDD Red+Green prompt" tdd)
+                 (const :tag "Use TDD Red+Green+Blue prompt" tdd-with-refactoring)
                  (const :tag "Ask every time" ask-me)
                  (const :tag "Off" nil))
   :set #'ai-code--test-after-code-change--set
@@ -413,6 +431,7 @@ Shows the current backend label to the right."
     ;; Use plist style to provide a dynamic description function.
     ("s" ai-code-select-backend :description ai-code--select-backend-description)
     ("u" "Install / Upgrade AI CLI" ai-code-upgrade-backend)
+    ("S" "Install skills for backend" ai-code-install-backend-skills)
     ("g" "Open backend config (eg. add mcp)" ai-code-open-backend-config)
     ("G" "Open backend repo agent file" ai-code-open-backend-agent-file)
     ("|" "Apply prompt on file" ai-code-apply-prompt-on-current-file)
@@ -455,15 +474,6 @@ Shows the current backend label to the right."
     ("N" "Toggle notifications" ai-code-notifications-toggle)]])
 
 
-
-;; DONE: When in an ai-coding session (e.g., with -backends-infra.el code) and using evil-mode,
-;; pressing SPC in normal state will trigger ai-code-send-command.
-;; Fixed implementation moved to ai-code-backends-infra.el:
-;;   - `ai-code-backends-infra--evil-spc-command' - intercepts SPC in AI session buffers
-;;   - `ai-code-backends-infra-evil-setup' - call with (with-eval-after-load 'evil ...)
-
-;; (with-eval-after-load 'evil (ai-code-backends-infra-evil-setup))
-
 (provide 'ai-code)
 
 ;;; ai-code.el ends here
